/**
 * @file Application.hpp
 * Application class - main class of "Highlight utility" application
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef APPLICATION_HPP
#define APPLICATION_HPP

#include <msgpack.hpp>
#include <string>

#include "HCEConstants.hpp"
#include "CrossThreadBuf.hpp"
#include "ComponentBase.hpp"
#include "ComponentExceptionBase.hpp"
#include "ComponentManager.hpp"
#include "ComponentManagerPool.hpp"
#include "ComponentManagerPoolIf.hpp"
#include "ComponentsFactory.hpp"
#include "ContentsStorageBase.hpp"
#include "EncodeDecodeBase64.hpp"
#include "InDataStruct.hpp"
#include "OutDataStruct.hpp"
#include "ServerApplication.hpp"

#define HIGHLIGHT_TIMEOUT_MLS_NAME "APP.highlightComponentTimeoutMLS"
#define REFINES_COUNT_NAME "APP.refinesCount"

class Application : public HCE::ServerApplication
{
  protected:
    bool exitFlag;
    HCE::component::ComponentsFactory componentsFactory;
    Poco::SharedPtr<HCE::component::ComponentManagerPoolIf> componentManagerPool;
    Poco::SharedPtr<CrossThreadBuf> crossThreadBufPtr;
    Poco::SharedPtr<ContentsStorageBase> contentsStoragePtr;
    std::string outDataString;
  protected:
    void factoriesRegister();
    void componentsInitialize();
    void addWCache(unsigned long long &resCrc, std::string &wCacheBase64);
    void extractWCache(msgpack::sbuffer &msgPackBuf, Poco::SharedPtr<OutDataStruct> outDataStructPtr);
    void highlightComponentExecutor(Poco::SharedPtr<InDataStruct> inDataStructPtr, Poco::SharedPtr<OutDataStruct> outDataStructPtr);
    virtual int main(const std::vector<std::string>& args);
    void readData(std::string &incomeString);
    bool checkNeededConfigParams();
    virtual void handleHelp(const std::string& name,const std::string& value);
    virtual void handleVersion(const std::string& name,const std::string& value);
  public:
    Application();
    virtual ~Application() throw();
};

#endif