/**
 * @file ContentsStorageBase.hpp
 * ContentsStorageBase - base class of content parse/storage processings.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef CONTENTS_STORAGE_BASE_HPP
#define CONTENTS_STORAGE_BASE_HPP

#include <Poco/Condition.h>
#include <Poco/Mutex.h>
#include <Poco/SharedPtr.h>

#include <vector>

#include "CRC64.h"
#include "CWords.hpp"
#include "InDataRefine.hpp"
#include "OutDataRefine.hpp"
#include "Refine.hpp"
#include "WordPos.hpp"

class ContentsStorageBase
{
  public:
    typedef Poco::SharedPtr<HCE::component::Refine> RefinePtrType;
//    typedef Poco::SharedPtr<std::map<unsigned long long, std::vector<WordPos> > > WordPosPtrType;
    typedef Poco::SharedPtr<std::vector<std::map<unsigned long long, std::vector<WordPos> > > > WordPosPtrType;
  private:
    Poco::FastMutex componentMutex;
    Poco::FastMutex refinesVectorMutex;
    Poco::Condition conditionVar;
    std::vector<std::pair<RefinePtrType, bool > > refinesVector;
  protected:
    RefinePtrType refineExctract();
    void refineRelease(RefinePtrType &refinePtr);
  public:
    ContentsStorageBase(unsigned int refinesCount);
    virtual WordPosPtrType wordMapExctract(unsigned long long key, std::string &incomeContent/*, unsigned char normalization, unsigned char subjectMask*/) = 0;
    virtual void wordMapRelease(unsigned long long key, WordPosPtrType &refinePtr) = 0;
    virtual ~ContentsStorageBase();
};

#endif