/**
 * @file ContentsStorageSimple.hpp
 * ContentsStorageBase - concrete class of content parse/storage processings, in simple mode.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef CONTENTS_STORAGE_SIMPLE_HPP
#define CONTENTS_STORAGE_SIMPLE_HPP

#include <atomic>
#include <map>
#include <unistd.h>

#include "ContentsStorageBase.hpp"

class ContentsStorageSimple : public ContentsStorageBase
{
  protected:
    void fillWordPosMap(Poco::SharedPtr<HCE::OutDataRefine> outDataRefine, ContentsStorageBase::WordPosPtrType ret);
    std::map<unsigned long long, std::pair<WordPosPtrType, Poco::SharedPtr<std::atomic_flag> > > wordPosCache;
  public:
    ContentsStorageSimple(unsigned int refinesCount);
    virtual WordPosPtrType wordMapExctract(unsigned long long key, std::string &incomeContent);
    virtual void wordMapRelease(unsigned long long key, WordPosPtrType &refinePtr);
    void cacheAddElement(unsigned long long key, WordPosPtrType wordPos);  
    void cacheClear();
    unsigned int getCacheSize() const {return wordPosCache.size();}
    virtual ~ContentsStorageSimple();
};

#endif