/**
 * @file CrossThreadBuf.hpp
 * CrossThreadBuf class implementation.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#include <stdexcept>

#include "CrossThreadBuf.hpp"

CrossThreadBuf::ResElement& CrossThreadBuf::getElem(unsigned int index) 
{
  if(index >= inOutData.size())
  {
    throw std::out_of_range(">>> CrossThreadBuf::getElem method");
  }
  return inOutData[index];  
}

std::string &CrossThreadBuf::getIntData(unsigned int &index, unsigned long long &id, unsigned long long &resCrc)
{
  Poco::ScopedLock<Poco::FastMutex> sLock(mutex);
  if(processingCount >= inOutData.size())
  {
    throw std::out_of_range(">>> CrossThreadBuf::getIntData method");
  }
  index = processingCount;
  id = inOutData[processingCount].getId();
  resCrc = inOutData[processingCount].getResCrc();
  return inOutData[processingCount++].getInData();
}

void CrossThreadBuf::setOutData(std::string &outData, unsigned int highlightCounter, unsigned int index)
{
  Poco::ScopedLock<Poco::FastMutex> sLock(mutex);
  if(index >= inOutData.size())
  {
    throw std::out_of_range(">>> CrossThreadBuf::setOutData method");
  }  
  inOutData[index].setOutData(outData);
  inOutData[index].setHighlightCounter(highlightCounter);
  finishedCount++;
}

void CrossThreadBuf::setTimeout(unsigned int _timeOut)
{
  tStart.update();
  timeOut = _timeOut;
}

/** CrossThreadBuf::getRestTimeout method.
 * Calcs and returs rest of timeout,
 * that was set in the previous setTimeout method call
 **/
unsigned int CrossThreadBuf::getRestTimeout()
{
  unsigned int ret = 0;
  Poco::ScopedLock<Poco::FastMutex> sLock(mutex);
  tCurr.update();
  Poco::Timestamp::TimeDiff tDiff = tCurr - tStart;
  if(tDiff < (timeOut * MICRO_SEC_MULTIPLIER))
  {
    ret = (timeOut * MICRO_SEC_MULTIPLIER - tDiff)/MICRO_SEC_MULTIPLIER;
  }
  return ret;
}

void CrossThreadBuf::waitCv()
{
  cvMutex.lock();
  cv.wait<Poco::FastMutex>(cvMutex);
  cvMutex.unlock();
}

void CrossThreadBuf::waitCv(long milliseconds)
{
  cvMutex.lock();
  try
  {
    cv.wait<Poco::FastMutex>(cvMutex, milliseconds);
  }
  catch(Poco::TimeoutException &excp)
  {
  }
  cvMutex.unlock();
}

void CrossThreadBuf::signal()
{
  cvMutex.lock();
  cv.signal();
  cvMutex.unlock();
}