/**
 * @file CrossThreadBuf.hpp
 * CrossThreadBuf class. It class shares data bsetween main application class and HiglightComponent class
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef CROSS_THREAD_BUF_HPP
#define CROSS_THREAD_BUF_HPP

#include <Poco/Condition.h>
#include <Poco/Mutex.h>
#include <Poco/Random.h>
#include <Poco/ScopedLock.h>
#include <Poco/SharedPtr.h>
#include <Poco/Timestamp.h>
#include <iostream>
#include <vector>

#include <msgpack.hpp>

#include "InDataStruct.hpp"

#define MICRO_SEC_MULTIPLIER 1000

class CrossThreadBuf
{
  public:
    class ResElement
    {
      private:
        std::string inData;
        std::string outData;
        unsigned int highlightCounter;
        unsigned long long id;
	unsigned long long resCrc;
	msgpack::sbuffer msgPackBuf;
      public:
	ResElement(const ResElement &element): inData(element.inData), outData(element.outData), highlightCounter(element.highlightCounter), id(element.id), resCrc(element.resCrc), msgPackBuf() {}
        ResElement():inData(""), outData(""), highlightCounter(0), id(0), resCrc(0), msgPackBuf() {}
        std::string &getInData() {return inData;}
        std::string &getOutData() {return outData;}
        unsigned int getHighlightCounter() const  {return highlightCounter;}
        unsigned long long getId() const  {return id;}
        unsigned long long getResCrc() const  {return resCrc;}
        msgpack::sbuffer &getMsgPackBuff() {return msgPackBuf;}
        ResElement& operator= (const ResElement& elem)
	{
	  inData = elem.inData;
	  outData = elem.outData;
	  highlightCounter = elem.highlightCounter;
	  id = elem.id;
	  resCrc = elem.resCrc;
	  msgPackBuf.write(elem.msgPackBuf.data(), elem.msgPackBuf.size());
	  return (*this);
	}
        void setInData(std::string _inData) {inData = _inData;}
        void setOutData(std::string _outData) {outData = _outData;}
        void setId(unsigned long long _id) {id = _id;}
        void setResCrc(unsigned long long _resCrc) {resCrc = _resCrc;}
        void setHighlightCounter(unsigned int _highlightCounter) {highlightCounter = _highlightCounter;}
        ~ResElement() {}
    };
  protected:
    Poco::FastMutex mutex;
    Poco::FastMutex cvMutex;
    Poco::Condition cv;
    Poco::Timestamp tStart;
    Poco::Timestamp tCurr;
    std::vector<ResElement> inOutData;
    unsigned int timeOut;
    unsigned int processingCount;
    unsigned int finishedCount;
    bool isSendWordsMapBack;
  public:
    CrossThreadBuf():timeOut(0), processingCount(0), finishedCount(0), isSendWordsMapBack(false) {}
    void addInOutData(ResElement &inOutDataElem) {inOutData.push_back(inOutDataElem);}
    unsigned int elemsCount() const {return inOutData.size();}
    bool isTaskFinished() const {return (finishedCount == inOutData.size());}
    void setIsSendWordsMapBack(bool _isSendWordsMapBack) {isSendWordsMapBack = _isSendWordsMapBack;}
    bool getIsSendWordsMapBack() const  {return isSendWordsMapBack;}
    ResElement& getElem(unsigned int index);
    std::string &getIntData(unsigned int &index, unsigned long long &id, unsigned long long &resCrc);
    void setOutData(std::string &outData, unsigned int highlightCounter, unsigned int index);
    void setTimeout(unsigned int _timeOut);
    unsigned int getRestTimeout();
    void waitCv();
    void waitCv(long milliseconds);
    void signal();
    void lock() {mutex.lock();}
    void unlock() {mutex.unlock();}
    ~CrossThreadBuf() {}
};
#endif