/**
 * @file ErrorStruct.cpp
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include "ErrorStruct.hpp"

std::vector<std::string> errorCodesStrings = {"ERR_Hlt_timeout", "ERR_Bad_regular_expr"};

ErrorStruct::ErrorStruct()
{
  errorMask = 0;
}

ErrorStruct::~ErrorStruct()
{
}

void ErrorStruct::addErrorCode(unsigned long long errorCode)
{
  mutex.lock();
  errorMask |= errorCode;
  mutex.unlock();
}

unsigned long long ErrorStruct::getErrorCode()
{
  unsigned long long ret = 0;
  mutex.lock();
  ret = errorMask;
  mutex.unlock();
  return ret;
}

std::string ErrorStruct::resolveErrorString(unsigned long long errorCode)
{
  for(unsigned int i = 0; i < (sizeof(errorCode) * 8); i++)
  {
    if(errorCode & LONG_LONG_MASK)
    {
      return errorCodesStrings[i];
    }
    errorCode >>= 1;
  }
  return "";
}

std::string ErrorStruct::resolveErrorString()
{
  return resolveErrorString(errorMask);
}

std::string ErrorStruct::concatenateErrorStrings(unsigned long long errorCode)
{
  std::string ret = "";
  for(unsigned int i = 0; i < (sizeof(errorCode) * 8); i++)
  {
    if(errorCode & LONG_LONG_MASK)
    {
      ret += errorCodesStrings[i];
      ret += SEPARATOR_STRING;
    }
    errorCode >>= 1;
  }
  if(ret.length() > 0)
  {
    ret.erase(ret.length() - SEPARATOR_STRING_LENGTH, SEPARATOR_STRING_LENGTH);
  }
  return ret;
}

std::string ErrorStruct::concatenateErrorStrings()
{
  return concatenateErrorStrings(errorMask);
}