/**
 * @file ErrorStruct.hpp
 * File contents headers and macros for Error processing
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#ifndef ERROR_CONSTANTS_HPP
#define ERROR_CONSTANTS_HPP

#include <string>
#include <vector>

#define LONG_LONG_MASK 0x0000000000000001llu
#define SEPARATOR_STRING " | "
#define SEPARATOR_STRING_LENGTH 3

#define ERROR_FLAG_WAS_TIMEOUT 0x1
#define ERROR_BAD_REG_EXPRESSION 0x2

extern std::vector<std::string> errorCodesStrings;

#include <Poco/Mutex.h>
  
struct ErrorStruct
{
  private:
    Poco::FastMutex mutex;
    unsigned long long errorMask;
  public:
    ErrorStruct();
    void addErrorCode(unsigned long long errorCode);
    unsigned long long getErrorCode();
    static std::string resolveErrorString(unsigned long long errorCode);
    std::string resolveErrorString();
    static std::string concatenateErrorStrings(unsigned long long errorCode);
    std::string concatenateErrorStrings();
    ~ErrorStruct();
};

#endif