/**
 * @file HighlightJsonWrapper.cpp
 * HighlightJsonWrapper class's implementation.
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#include <Poco/JSON/JSONException.h>
#include <Poco/JSON/Stringifier.h>

#include "HighlightJsonException.hpp"
#include "HighlightJsonWrapper.hpp"
#include "SearchStringParser.hpp"

const std::string HighlightJsonWrapper::QUERY_NAME = "q";
const std::string HighlightJsonWrapper::CONTENTS_NAME = "contents";
const std::string HighlightJsonWrapper::PARAMETERS_NAME = "parameters";
const std::string HighlightJsonWrapper::RETURN_HASH_NAME = "return_hash";
                                                        
const std::string HighlightJsonWrapper::PARAM_BRGIN_MARKER_NAME = "begin_marker";
const std::string HighlightJsonWrapper::PARAM_END_MARKER_NAME = "end_marker";
const std::string HighlightJsonWrapper::PARAM_MAX_NUMBER_NAME = "max_number";
const std::string HighlightJsonWrapper::PARAM_DELIMITERS_NAME = "delimiters";
const std::string HighlightJsonWrapper::PARAM_ACRONYMS_NAME = "acronyms";
const std::string HighlightJsonWrapper::PARAM_NUMBERS_AS_WORDS_NAME = "numbers_as_words";
const std::string HighlightJsonWrapper::PARAM_USE_SINGLE_CHARS_NAME = "use_single_chars";
const std::string HighlightJsonWrapper::PARAM_TIMEOUT = "timeout";
//----------------------- OutPut Params names --------------------------------------------------------------------------------------
const std::string HighlightJsonWrapper::PARAM_ERROR_CODE_NAME = "error_code";
const std::string HighlightJsonWrapper::PARAM_ERROR_MSG_NAME = "error_msg";
const std::string HighlightJsonWrapper::PARAM_CONTENTS_NAME = "contents";
const std::string HighlightJsonWrapper::PARAM_HIGHLIGHTTED_NAME = "highlighted";
const std::string HighlightJsonWrapper::PARAM_FOUND_NAME = "found";
const std::string HighlightJsonWrapper::PARAM_TOTAL_TIME_NAME = "time";

using namespace HCE::exception;

Poco::Dynamic::Var HighlightJsonWrapper::extractParam(const std::string& paramName, Poco::JSON::Object::Ptr parametersPtr)
{
  std::vector<Poco::Dynamic::Var> varPath;
  Poco::Dynamic::Var ret;
  std::string errStr;

  varPath.push_back(Poco::Dynamic::Var(paramName));
  ret = jsonObjExtractor.getAsVar(parametersPtr, varPath);
  if(ret.isEmpty())
  {
    errStr = paramName;
    errStr += " PARAM NOT FOUND";
    throw HighlightJsonException(errStr);
  }
  return ret;
}

void HighlightJsonWrapper::parseParams(Poco::SharedPtr<InDataStruct> inDataStructPtr, Poco::JSON::Object::Ptr parametersPtr)
{
  InDataStruct::InternalInDataParams inDataParams;
  Poco::Dynamic::Var localVar;
  
  localVar = extractParam(PARAM_BRGIN_MARKER_NAME, parametersPtr);
  inDataParams.beginMarker = localVar.extract<std::string>();

  localVar = extractParam(PARAM_END_MARKER_NAME, parametersPtr);
  inDataParams.endMarker = localVar.extract<std::string>();

  localVar = extractParam(PARAM_MAX_NUMBER_NAME, parametersPtr);
  inDataParams.maxNumber = localVar.extract<int>();
  
  localVar = extractParam(PARAM_DELIMITERS_NAME, parametersPtr);
  inDataParams.delimiters = localVar.extract<std::string>();

  localVar = extractParam(PARAM_ACRONYMS_NAME, parametersPtr);
  inDataParams.acronyms = localVar.extract<std::string>();
  
  localVar = extractParam(PARAM_NUMBERS_AS_WORDS_NAME, parametersPtr);
  inDataParams.numbersAsWords = localVar.extract<bool>();  
  
  localVar = extractParam(PARAM_USE_SINGLE_CHARS_NAME, parametersPtr);
  inDataParams.useSingleChars = localVar.extract<bool>();  
  
  localVar = extractParam(PARAM_TIMEOUT, parametersPtr);
  inDataParams.timeout = localVar.extract<int>();  
  
  inDataStructPtr->addParams(inDataParams);
}

Poco::SharedPtr<InDataStruct> HighlightJsonWrapper::inDataFromJsomString(const std::string &jsonString)
{
  Poco::JSON::Parser parser;
  Poco::SharedPtr<InDataStruct> ret = nullptr;
  Poco::JSON::Object::Ptr obj = nullptr;
  std::string localQueryStr;
  Poco::Dynamic::Var result;
  try
  {
    result = parser.parse(jsonString);
  }
  catch(Poco::JSON::JSONException &excp)
  {
    throw HighlightJsonException("PARSE JSON EXCEPTION --- " + excp.message());
  }
  catch(Poco::SyntaxException  &excp)
  {
    throw HighlightJsonException("PARSE JSON EXCEPTION --- " + excp.message());
  }  
  std::string errStr;
  if(result.type() == typeid(Poco::JSON::Object::Ptr))
  {
    try
    {
      obj = result.extract<Poco::JSON::Object::Ptr>();
    }
    catch(Poco::BadCastException &excp)
    {
      throw HighlightJsonException("CANT'T CAST JSON RESULT TO THE [Poco::JSON::Object::Ptr]");
    }
  }
  if(obj)
  {
    std::vector<Poco::Dynamic::Var> varPath;
    varPath.push_back(Poco::Dynamic::Var(QUERY_NAME));    
    Poco::Dynamic::Var queryStr = jsonObjExtractor.getAsVar(obj, varPath);
    if(queryStr.isEmpty())
    {
      errStr = "EMPTY OR NOT FOUND [";
      errStr += QUERY_NAME;
      errStr += "] PARAM";
      throw HighlightJsonException(errStr);
    }

    varPath.clear();
    varPath.push_back(Poco::Dynamic::Var(CONTENTS_NAME));
    Poco::JSON::Array::Ptr contentArrayPtr = jsonObjExtractor.getAsArray(obj, varPath);
    if(contentArrayPtr.isNull())
    {
      errStr = "EMPTY OR NOT FOUND, OR TYPE NOT ARRAY [";
      errStr += CONTENTS_NAME;
      errStr += "] PARAM";
      throw HighlightJsonException(errStr);
    }

    varPath.clear();
    varPath.push_back(Poco::Dynamic::Var(PARAMETERS_NAME));
    Poco::JSON::Object::Ptr parametersPtr = jsonObjExtractor.getAsObject(obj, varPath);
    if(parametersPtr.isNull())
    {
      errStr = "EMPTY OR NOT FOUND, OR TYPE NOT OBJECT [";
      errStr += PARAMETERS_NAME;
      errStr += "] PARAM";
      throw HighlightJsonException(errStr);
    }

    varPath.clear();
    varPath.push_back(Poco::Dynamic::Var(RETURN_HASH_NAME));
    Poco::Dynamic::Var returnHashVar = jsonObjExtractor.getAsVar(obj, varPath);
    if(returnHashVar.isEmpty())
    {
      errStr = "EMPTY OR NOT FOUND[";
      errStr += RETURN_HASH_NAME;
      errStr += "] PARAM";
      throw HighlightJsonException(errStr);
    }
    
    ret = Poco::SharedPtr<InDataStruct>(new InDataStruct());
    ret->setSendCacheBack(returnHashVar.extract<bool>());
    try
    {
      parseParams(ret, parametersPtr);
    }
    catch(Poco::BadCastException &excp)
    {
      errStr = "CAN'T CAST SOME PARAM'S TYPE, CHECK THEIR TYPES";
      throw HighlightJsonException(errStr);
    }

    Poco::JSON::Array::Ptr arrayVal;
    Poco::Dynamic::Var arraySubVal;
    for(unsigned int i = 0; i < contentArrayPtr->size(); i++)
    {
      arrayVal = contentArrayPtr->getArray(i);
      if(!arrayVal.isNull())
      {
	try
	{
	  arraySubVal = arrayVal->get(0);
          const std::string &constContentStr = arraySubVal.extract<std::string>();
          ret->addContent(const_cast<std::string &>(constContentStr));
	  arraySubVal = arrayVal->get(1);
          const std::string &constWcacheStr = arraySubVal.extract<std::string>();
          ret->addWcache(const_cast<std::string &>(constWcacheStr));
	}
	catch(Poco::BadCastException &excp)
	{
          errStr = "CAN'T CAST CONTENT SUB_ARRAY ELEM [";
          errStr += std::to_string(i);
          errStr += "] TO STRING";
          throw HighlightJsonException(errStr);		  
	}
      }
      else
      {
        errStr = "CAN'T CAST CONTENT ARRAY ELEM [";
        errStr += std::to_string(i);
        errStr += "] TO STRING";
        throw HighlightJsonException(errStr);	
      }  
    }
    try
    {
      localQueryStr = queryStr.extract<std::string>();
    }
    catch(Poco::BadCastException &excp)
    {
      throw HighlightJsonException("CAN'T CAST QUERY TO STRING");
    }
    std::vector<std::vector<std::pair<SearchType, std::string> > > searchStringsVector = SearchStringParser::parse(localQueryStr, SEARCH_STRING_SEPARATOR);
    if(searchStringsVector.size() == 0)
    {
      throw HighlightJsonException("CAN'T EXTRACT SEARCH STRINGS FROM QUERY STRING");
    }
    ret->setSearchStrings(searchStringsVector);
  }
  return ret;
}

void HighlightJsonWrapper::jsonStringFromOutData(Poco::SharedPtr<OutDataStruct> outDataStructPtr, std::string &outJSONString, Poco::Timestamp::TimeDiff tDiff)
{
  Poco::Dynamic::Var var;
  Poco::Dynamic::Var val; 
  Poco::JSON::Array::Ptr contentArray = Poco::JSON::Array::Ptr(new Poco::JSON::Array());
  Poco::JSON::Array::Ptr contentSubArray = nullptr;
  Poco::JSON::Object::Ptr obj = Poco::JSON::Object::Ptr(new Poco::JSON::Object());
  val = outDataStructPtr->getErrCode();
  obj->set(PARAM_ERROR_CODE_NAME, val);
  val = outDataStructPtr->getErrorMsg();
  obj->set(PARAM_ERROR_MSG_NAME, val);
  val = outDataStructPtr->getHighlightedCounter();
  obj->set(PARAM_HIGHLIGHTTED_NAME, val);
  val = outDataStructPtr->getFoundCounter();
  obj->set(PARAM_FOUND_NAME, val);
  obj->set(PARAM_TOTAL_TIME_NAME, static_cast<unsigned int>(tDiff));
  
  for(unsigned int i = 0; i < outDataStructPtr->getContentsCount(); i++)
  {
    contentSubArray = Poco::JSON::Array::Ptr(new Poco::JSON::Array());
    val = outDataStructPtr->getContent(i);
    contentSubArray->add(val);
    if(i < outDataStructPtr->getWcachesCount())
    {
      val = outDataStructPtr->getWcache(i);
    }
    else
    {
      val = std::string("");
    }
    contentSubArray->add(val);
    contentArray->add(contentSubArray);
    contentSubArray = nullptr; 
  }

  obj->set(PARAM_CONTENTS_NAME, contentArray);
  var = obj;
  std::ostringstream outJSON;
  Poco::JSON::Stringifier::stringify(var, outJSON, 1);
  outJSONString = outJSON.str();
}