/**
 * @file HighlightJsonWrapper.hpp
 * HighlightJsonWrapper class that convert/parse incoming JSON string to InDataStruct struct
 * and back OutDataStruct struct to the outgoing JSON string.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#ifndef HIGHLIGHT_JSON_WRAPPER_HPP
#define HIGHLIGHT_JSON_WRAPPER_HPP

#include <Poco/JSON/Parser.h>
#include <Poco/SharedPtr.h>
#include <Poco/Timestamp.h>

#include "InDataStruct.hpp"
#include "JsonObjExtractor.hpp"
#include "OutDataStruct.hpp"

#define SEARCH_STRING_SEPARATOR "::"

using namespace HCE::utils;

class HighlightJsonWrapper
{
  protected:
    JsonObjExtractor jsonObjExtractor;
  public:
//----------------------- InPut Param names --------------------------------------------------------------------------------------    
    static const std::string QUERY_NAME;
    static const std::string CONTENTS_NAME;
    static const std::string PARAMETERS_NAME;
    static const std::string RETURN_HASH_NAME;

    static const std::string PARAM_BRGIN_MARKER_NAME;
    static const std::string PARAM_END_MARKER_NAME;
    static const std::string PARAM_MAX_NUMBER_NAME;
    static const std::string PARAM_DELIMITERS_NAME;
    static const std::string PARAM_ACRONYMS_NAME;
    static const std::string PARAM_NUMBERS_AS_WORDS_NAME;
    static const std::string PARAM_USE_SINGLE_CHARS_NAME;
    static const std::string PARAM_TIMEOUT;
//----------------------- OutPut Param names --------------------------------------------------------------------------------------
    static const std::string PARAM_ERROR_CODE_NAME;
    static const std::string PARAM_ERROR_MSG_NAME;
    static const std::string PARAM_CONTENTS_NAME;
    static const std::string PARAM_HIGHLIGHTTED_NAME;
    static const std::string PARAM_FOUND_NAME;
    static const std::string PARAM_TOTAL_TIME_NAME;
  protected:
    Poco::Dynamic::Var extractParam(const std::string& paramName, Poco::JSON::Object::Ptr parametersPtr);
    void parseParams(Poco::SharedPtr<InDataStruct> inDataStructPtr, Poco::JSON::Object::Ptr parametersPtr);
  public:
    HighlightJsonWrapper() {}
    Poco::SharedPtr<InDataStruct> inDataFromJsomString(const std::string &jsonString);
// cppcheck-suppress *
    void jsonStringFromOutData(Poco::SharedPtr<OutDataStruct> outDataStructPtr, std::string &outJSONString, Poco::Timestamp::TimeDiff tDiff);
    virtual ~HighlightJsonWrapper() {}
};

#endif
