/**
 * @file HighlightStringGenerator.hpp
 * HighlightStringGenerator class gets : content string, words position Map, search string 
 * and results highlight content.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#ifndef HIGHLIGHT_STRING_GENERATOR_HPP
#define HIGHLIGHT_STRING_GENERATOR_HPP

#include <Poco/SharedPtr.h>
#include <Poco/UTF8String.h>
#include <map>
#include <string>
#include <vector>

#include "ContentsStorageBase.hpp"
#include "CRC64.h"
#include "CWords.hpp"
#include "InDataRefine.hpp"
#include "InDataStruct.hpp"
#include "HighlightingAlgorithmBase.hpp"
#include "HighlightStringStickerBase.hpp"
#include "OutDataRefine.hpp"
#include "Refine.hpp"
#include "WordPos.hpp"

class HighlightStringGenerator
{
  protected:
    unsigned int maxNumber;
    ContentsStorageBase::WordPosPtrType wordMapPtr;
    Poco::SharedPtr<std::string> contentPtr;
    std::string hlBegin;
    std::string hlEnd;
    std::vector<Poco::SharedPtr<HighlightingAlgorithmBase> > highlightingAlgorithms;
    Poco::SharedPtr<HighlightStringStickerBase> highlightStringStickerPtr;
  protected:
    std::vector<WordPos> stickWordOffsetMaps(std::vector<std::map<unsigned int, WordPos> > &wordOffsetMaps);
// cppcheck-suppress *    
    void joinOverlapElements(std::map<unsigned int, WordPos> &resultingMap, std::vector<WordPos> &ret);
  public:
    HighlightStringGenerator();
    inline void setMaxNumber(unsigned int _maxNumber);    
    inline void setWordMapPtr(ContentsStorageBase::WordPosPtrType _wordMapPtr);
    inline void setContent(Poco::SharedPtr<std::string> _contentPtr);
    inline void setHlBegin(std::string _hlBegin);
    inline void setHlEnd(std::string _hlEnd);
    inline void addHighlightingAlgorithm(Poco::SharedPtr<HighlightingAlgorithmBase> _highlightingAlgorithm);
    inline void setHighlightStringSticker(Poco::SharedPtr<HighlightStringStickerBase> _highlightStringStickerPtr);
    std::string highlightStringGenerate(std::vector<std::vector<std::pair<SearchType, std::string> > > &vecSearchStrings);
    inline unsigned int getHighlightCounts();
    inline void highlightStringStickerReinit(Poco::SharedPtr<InDataStruct> inDataStructPtr);
    std::string generateHighlightingAlgorithmsString(const std::string &separator);
    ~HighlightStringGenerator();
};

void HighlightStringGenerator::setMaxNumber(unsigned int _maxNumber)
{
  maxNumber = _maxNumber;
}

void HighlightStringGenerator::setWordMapPtr(ContentsStorageBase::WordPosPtrType _wordMapPtr)
{
  wordMapPtr = _wordMapPtr;
}

void HighlightStringGenerator::setContent(Poco::SharedPtr<std::string> _contentPtr)
{
  contentPtr = _contentPtr;
}

void  HighlightStringGenerator::setHlBegin(std::string _hlBegin)
{
  hlBegin = _hlBegin;
}

void  HighlightStringGenerator::setHlEnd(std::string _hlEnd)
{
  hlEnd = _hlEnd;
}

void HighlightStringGenerator::addHighlightingAlgorithm(Poco::SharedPtr<HighlightingAlgorithmBase> _highlightingAlgorithm)
{
  highlightingAlgorithms.push_back(_highlightingAlgorithm);
}

void HighlightStringGenerator::setHighlightStringSticker(Poco::SharedPtr<HighlightStringStickerBase> _highlightStringStickerPtr)
{
  highlightStringStickerPtr = _highlightStringStickerPtr;
}

unsigned int HighlightStringGenerator::getHighlightCounts()
{
  if(!highlightStringStickerPtr.isNull())
  {
    return highlightStringStickerPtr->getHighlightedCounter();
  }
  else
  {
    return 0;
  }
}

void HighlightStringGenerator::highlightStringStickerReinit(Poco::SharedPtr<InDataStruct> inDataStructPtr)
{
  if(!highlightStringStickerPtr.isNull())
  {
    highlightStringStickerPtr->reinitialize(inDataStructPtr);
  }
}

#endif