/**
 * @file HighlightStringStickerSimple.cpp
 * HighlightStringStickerSimple class implementation.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <Poco/SharedPtr.h>

#include "HighlightStringStickerSimple.hpp"

HighlightStringStickerSimple::HighlightStringStickerSimple():hlBegin(""), hlEnd(""), maxNumber(0xFFFFFFFF), highlightedCounter(0)
{
}

std::string HighlightStringStickerSimple::fillOutPutStr(std::vector<WordPos> &wordOffsets, const std::string &incomeContent)
{
  std::string ret;
  unsigned int endPos = 0;
  for(unsigned int i = 0; i < wordOffsets.size(); i++)
  {
    if(wordOffsets[i].beginOffset > incomeContent.length() || wordOffsets[i].endOffset > incomeContent.length())
    {
      throw std::length_error("bad string length");
    }
    highlightedCounter++;
    ret.append(incomeContent.begin() + endPos, incomeContent.begin() + wordOffsets[i].beginOffset);
    ret.append(hlBegin);
    ret.append(incomeContent.begin() + wordOffsets[i].beginOffset, incomeContent.begin() + wordOffsets[i].endOffset);
    ret.append(hlEnd);
    endPos = wordOffsets[i].endOffset;
    if(highlightedCounter == maxNumber)
    {
      break;
    }
  }
  ret.append(incomeContent.begin() + endPos, incomeContent.end());
  return ret;
}

void HighlightStringStickerSimple::reinitialize(Poco::SharedPtr<InDataStruct> inDataStructPtr)
{
  if(!inDataStructPtr.isNull())
  {
    setHlBegin(inDataStructPtr->getParams(0).beginMarker);
    setHlEnd(inDataStructPtr->getParams(0).endMarker);
    setMaxNumber(inDataStructPtr->getParams(0).maxNumber);
  }
}

void HighlightStringStickerSimple::setHighlightedCounter(unsigned int _highlightedCounter)
{
  highlightedCounter = _highlightedCounter;
}

unsigned int HighlightStringStickerSimple::getHighlightedCounter()
{
  return highlightedCounter;
}

HighlightStringStickerSimple::~HighlightStringStickerSimple()
{
}