/**
 * @file HighlightingAlgorithmBase.hpp
 *
 * abstract class HighlightingAlgorithmBase. It contains interface for HighlightingAlgorithm classes family
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef HIGHLIGHTING_ALGORITHM_BASE_HPP
#define HIGHLIGHTING_ALGORITHM_BASE_HPP

#include <map>
#include <Poco/SharedPtr.h>
#include <string>
#include <vector>

#include "ContentsStorageBase.hpp"
#include "InDataStruct.hpp"
#include "WordPos.hpp"


class HighlightingAlgorithmBase
{
  protected:
    HighlightingAlgorithmBase() {}
  public:
    virtual void process(const std::vector<std::pair<SearchType, std::string> > &searchStrings, const Poco::SharedPtr<std::string> contentPtr,
                 ContentsStorageBase::WordPosPtrType wordMapPtr, std::map<unsigned int, WordPos> &wordOffsetMap) = 0;
    virtual std::string getName() = 0;
    virtual ~HighlightingAlgorithmBase() {}
};

#endif