/**
 * @file HighlightingAlgorithmRefine.hpp
 *
 * Class HighlightingAlgorithmRefine. It contains Highlighting Algoritm implementation through "Refine" component
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef HIGHLIGHTING_ALGORITHM_REFINE_HPP
#define HIGHLIGHTING_ALGORITHM_REFINE_HPP

#include "CRC64.h"
#include "CWords.hpp"
#include "InDataRefine.hpp"
#include "HighlightingAlgorithmBase.hpp"
#include "NotStricktChain.hpp"
#include "OutDataRefine.hpp"
#include "Refine.hpp"

class HighlightingAlgorithmRefine : public HighlightingAlgorithmBase
{
  private:
    unsigned char subjectMask;
    unsigned int incrementalIndex;
    HCE::component::Refine refine;
    std::map<unsigned int, std::pair<unsigned long long, WordPos> > wordPositionMap;
    std::string name;
  protected:
    void setWordPositionMap(std::map<unsigned int, std::pair<unsigned long long, WordPos> > &_wordPositionMap);
    void fillWordOffsetMap(Poco::SharedPtr<HCE::OutDataRefine> outData, SearchType searchType,
                           std::map<unsigned long long, std::vector<WordPos> > &wordMapPtr, std::map<unsigned int, WordPos> &wordOffsetMap);
    void findChain(Poco::SharedPtr<HCE::OutDataRefine> outData, std::map<unsigned int, WordPos> &wordOffsetMap);
    void findChainStrict(Poco::SharedPtr<HCE::OutDataRefine> outData, std::map<unsigned int, WordPos> &wordOffsetMap);
    void joinBySegmentId(std::map<unsigned int, WordPos> &wordOffsetMap);
  public:
    HighlightingAlgorithmRefine():subjectMask(0), incrementalIndex(0), name("(A-HCE::Refine)") {}
    HighlightingAlgorithmRefine(unsigned char _normalizationId, unsigned char _subjectMask): subjectMask(_subjectMask), incrementalIndex(0), name("(A-HCE::Refine)") {}
    virtual void process(const std::vector<std::pair<SearchType, std::string> > &searchStrings, const Poco::SharedPtr<std::string> contentPtr, 
			 ContentsStorageBase::WordPosPtrType wordMapPtr, std::map<unsigned int, WordPos> &wordOffsetMap);
    virtual std::string getName() {return name;}
    virtual ~HighlightingAlgorithmRefine() {}
};

#endif