/**
 * @file HighlightingAlgorithmRegexpr.cpp
 *
 * class HighlightingAlgorithmRegexpr implementation.
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <Poco/RegularExpression.h>

#include "ErrorStruct.hpp"
#include "HighlightingAlgorithmRegexpr.hpp"

#include <exception>
#include <iostream>

extern ErrorStruct errorStruct;

void HighlightingAlgorithmRegexpr::process(const std::vector<std::pair<SearchType, std::string> > &searchStrings, const Poco::SharedPtr<std::string> contentPtr,
                                   ContentsStorageBase::WordPosPtrType wordMapPtr, std::map<unsigned int, WordPos> &wordOffsetMap)
{
  if(searchStrings.size() > 0 && !contentPtr.isNull() && !contentPtr->empty())
  {
    Poco::RegularExpression::Match match;
    unsigned int segmentId = 0;
    for(unsigned int i = 0; i < searchStrings.size(); i++)
    {
      if(!searchStrings[i].second.empty())
      {
	try
	{
//	  std::string::size_type offset = 0;
	  Poco::RegularExpression rexp(searchStrings[i].second, 0, false);
//	  int matchPos = rexp.match((*contentPtr), offset, match);
	  while(match.offset != std::string::npos)
	  {
	    auto it = wordOffsetMap.find(match.offset);
	    if(it == wordOffsetMap.end())
	    {
	      wordOffsetMap.insert(std::pair<unsigned int, WordPos>(match.offset, WordPos(match.offset, match.offset + match.length, match.offset, segmentId)));
	    }
	    else if(it->second.endOffset < (match.offset + match.length))
	    {
	      it->second.endOffset = (match.offset + match.length);
	      it->second.segmentId = segmentId;
	    }
	    segmentId++;	    
//	    offset = match.offset + match.length;
//	    matchPos = rexp.match((*contentPtr), offset, match);
	  }
	}
	catch(Poco::RegularExpressionException &excp)
	{
	  errorStruct.addErrorCode(ERROR_BAD_REG_EXPRESSION);
	}	
      }
    }
  }
}