/**
 * @file HighlightingAlgorithmSimple.hpp
 *
 * Class HighlightingAlgorithmSimple implementation.
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#include <Poco/UTF8String.h>

#include "HighlightingAlgorithmSimple.hpp"

/**
 * Method for simple substring search, that method(function) have called if main algorithm doesn't return any results.
**/
void HighlightingAlgorithmSimple::process(const std::vector<std::pair<SearchType, std::string> > &searchStrings, const Poco::SharedPtr<std::string> contentPtr,
                                  ContentsStorageBase::WordPosPtrType wordMapPtr, std::map<unsigned int, WordPos> &wordOffsetMap)
{
  if(contentPtr.isNull() || contentPtr->empty())
  {
    return;
  }  
  unsigned int fakePosition = 0;
// cppcheck-suppress variableScope  
  size_t findPos = 0;
  const std::string lowerCaseContent = Poco::UTF8::toLower(*contentPtr);
  WordPos localWpos;
  for(unsigned int i = 0; i < searchStrings.size(); i++)
  {
    findPos = 0;
    const std::string lowerCaseSearchString = Poco::UTF8::toLower(searchStrings[i].second);
    while((findPos = lowerCaseContent.find(lowerCaseSearchString, findPos)) != std::string::npos)
    {
      fakePosition = static_cast<unsigned int>(findPos);
      auto it = wordOffsetMap.find(fakePosition);
      if(it != wordOffsetMap.end())
      {
	if(((*it).second.endOffset - (*it).second.beginOffset) < lowerCaseSearchString.length())
	{
	  (*it).second.endOffset = ((*it).second.beginOffset + lowerCaseSearchString.length());
	}
      }
      else
      {
	localWpos.beginOffset = fakePosition;
	localWpos.endOffset = fakePosition + lowerCaseSearchString.length();
	localWpos.position = fakePosition;
	localWpos.segmentId++;
	wordOffsetMap.insert(std::pair<unsigned int, WordPos>(fakePosition, localWpos));
      }
      findPos++;
      if(wordOffsetMap.size() == maxNumber)
      {
	break;
      }
    }
    if(wordOffsetMap.size() == maxNumber)
    {
      break;
    }
  }  
}