/**
 * @file InDataHighlight.hpp
 *
 * class InDataHighlight. Incoming data structure for HiglightComponent
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef IN_DATA_HIGHLIGHT_HPP
#define IN_DATA_HIGHLIGHT_HPP

#include <string>

#include <Poco/SharedPtr.h>

#include "HCEConstants.hpp"
#include "DataBase.hpp"
#include "ContentsStorageBase.hpp"
#include "CrossThreadBuf.hpp"
#include "InDataStruct.hpp"

namespace HCE
{
  class InDataHighlight: public DataBase
  {
    private:
      Poco::SharedPtr<CrossThreadBuf> crossThreadBufPtr;
      Poco::SharedPtr<InDataStruct> inDataStructPtr;
      Poco::SharedPtr<ContentsStorageBase> contentsStorageBasePtr;
    public:
      InDataHighlight(ComponentType inType) : DataBase(inType), crossThreadBufPtr(nullptr), inDataStructPtr(nullptr), contentsStorageBasePtr(nullptr) {}
      void setCrossThreadBufPtr(Poco::SharedPtr<CrossThreadBuf> _crossThreadBufPtr) {crossThreadBufPtr = _crossThreadBufPtr;}
      Poco::SharedPtr<CrossThreadBuf> getCrossThreadBufPtr() {return crossThreadBufPtr;}
      void setInDataStructPtr(Poco::SharedPtr<InDataStruct> _inDataStructPtr) {inDataStructPtr = _inDataStructPtr;}
      Poco::SharedPtr<InDataStruct> getInDataStructPtr() {return inDataStructPtr;}
      void setContentsStorageBasePtr(Poco::SharedPtr<ContentsStorageBase> _contentsStorageBasePtr) {contentsStorageBasePtr = _contentsStorageBasePtr;}
      Poco::SharedPtr<ContentsStorageBase> getContentsStorageBasePtr() {return contentsStorageBasePtr;}
      virtual ~InDataHighlight() {crossThreadBufPtr = nullptr;}
  };
}

#endif