/**
 * @file InDataStruct.hpp
 *
 * class InDataStruct definition. It class contents "Highlight" unil's incoming data.
 * Also here fefine SearchType enum, that represents 3 search type.
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef ID_DATA_STRUCT_HPP
#define ID_DATA_STRUCT_HPP

#include <stdexcept>
#include <string>
#include <vector>

enum class SearchType : std::uint8_t {ST_SINGLE = 0, ST_STRICKT = 1, ST_NO_STRICKT = 2};

class InDataStruct
{
  public:
    struct InternalInDataParams
    {
      std::string beginMarker;
      std::string endMarker;
      unsigned int maxNumber;
      std::string delimiters;
      std::string acronyms;
      bool numbersAsWords;
      bool useSingleChars;
      unsigned int timeout;
      InternalInDataParams():beginMarker(""), endMarker(""), maxNumber(0xFFFFFFFF), delimiters(""), acronyms(""), numbersAsWords(false), useSingleChars(false), timeout(0)
      {
      }
    };
  private:
    std::vector<std::vector<std::pair<SearchType, std::string> > > searchStrings;
    std::vector<std::string> contents;
    std::vector<std::string> wcaches;
    std::vector<InternalInDataParams> paramsList;
    bool sendCacheBack;
  public:
    InDataStruct():sendCacheBack(false) {}
    void setSearchStrings(std::vector<std::vector<std::pair<SearchType, std::string> > > &_searchStrings)
    {
      searchStrings = _searchStrings;
    }
    void addContent(std::string &content)
    {
      contents.push_back(content);
    }
    void addContent(std::string &&content)
    {
      contents.push_back(content);
    }
    void addWcache(std::string &wcache)
    {
      wcaches.push_back(wcache);
    }
    void addWcache(std::string &&wcache)
    {
      wcaches.push_back(wcache);
    }
    void addParams(InternalInDataParams &params)
    {
      paramsList.push_back(params);
    }
    void setSendCacheBack(bool _sendCacheBack)
    {
      sendCacheBack = _sendCacheBack;
    }
    
    std::vector<std::vector<std::pair<SearchType, std::string> > > &getSearchStrings()
    {
      return searchStrings;
    }
    std::string &getContent(unsigned int i)
    {
      if(contents.size() <= i)
      {
        throw std::runtime_error(">>> Index out of range");
      }
      return contents[i];
    }
    std::string &getWcache(unsigned int i)
    {
      if(wcaches.size() <= i)
      {
        throw std::runtime_error(">>> Index out of range");
      }
      return wcaches[i];
    }
    InternalInDataParams &getParams(unsigned int i)
    {
      if(paramsList.size() <= i)
      {
        throw std::runtime_error(">>> Index out of range");
      }
      return paramsList[i];
    }
    unsigned int getContentsCount() const
    {
      return contents.size();
    }
    unsigned int getParamsCount() const
    {
      return paramsList.size();
    }
    unsigned int getWcachesCount() const
    {
      return wcaches.size();
    }    
    bool getSendCacheBack() const
    {
      return sendCacheBack;
    }

    ~InDataStruct() {}
};

#endif