/**
 * @file NotStricktChain.hpp
 *
 * class NotStricktChain. Found strickt word chains
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef NOT_STRICKT_CHAIN_HPP
#define NOT_STRICKT_CHAIN_HPP

#include <map>

//#include "WordPos.hpp"

class NotStricktChain
{
  protected:
    std::map<unsigned long long, std::pair<unsigned int, unsigned int> > wMap;
  public:
    NotStricktChain() {}
    inline void addWord(unsigned long long wordId);
    inline bool checkOnZerro();
    inline void setDefaultCounts();
    inline void decrCount(unsigned long long wordId);
    inline unsigned int wCount() const;
    ~NotStricktChain() {}
};

void NotStricktChain::addWord(unsigned long long wordId)
{
  typename std::map<unsigned long long, std::pair<unsigned int, unsigned int> >::iterator it;
  if((it = wMap.find(wordId)) == wMap.end())
  {
    wMap.insert(std::pair<unsigned long long, std::pair<unsigned int, unsigned int> >(wordId, std::pair<unsigned int, unsigned int>(1, 0)));
  }
  else
  {
    (*it).second.first++;
  }
}

bool NotStricktChain::checkOnZerro()
{
  bool ret = true;
  typename std::map<unsigned long long, std::pair<unsigned int, unsigned int> >::iterator it;
  for(it = wMap.begin(); it != wMap.end(); ++it)
  {
    if((*it).second.second != 0)
    {
      ret = false;
      break;
    }
  }
  return ret;
}

void NotStricktChain::setDefaultCounts()
{
  typename std::map<unsigned long long, std::pair<unsigned int, unsigned int> >::iterator it;
  for(it = wMap.begin(); it != wMap.end(); ++it)
  {
    (*it).second.second = (*it).second.first;
  }
}

void NotStricktChain::decrCount(unsigned long long wordId)
{
  typename std::map<unsigned long long, std::pair<unsigned int, unsigned int> >::iterator it;
  if((it = wMap.find(wordId)) != wMap.end())
  {
    (*it).second.second--;
  }  
}

unsigned int NotStricktChain::wCount() const
{
  return wMap.size();
}
#endif