/**
 * @file OutDataStruct.hpp
 *
 * class OutDataStruct definition. It class contents "Highlight" unil's outgoing data.
 * 
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#ifndef OUT_DATA_STRUCT_HPP
#define OUT_DATA_STRUCT_HPP

#include <stdexcept>
#include <string>
#include <vector>

class OutDataStruct
{
  private:
    unsigned int errorCode;
    std::string errorMsg;
    std::vector<std::string> contents;
    std::vector<std::string> wcaches;
    unsigned int highlightedCounter;
    unsigned int foundCounter;
  public:
    OutDataStruct():errorCode(0), errorMsg(""), highlightedCounter(0), foundCounter(0) {}
    void setErrorCode(unsigned int _errorCode) 
    {
      errorCode = _errorCode;
    }
    void setErrorMsg(const std::string &_errorMsg)
    {
      errorMsg = _errorMsg;
    }
    void addContent(std::string &content)
    {
      contents.push_back(content);
    }
    void addContent(std::string &&content)
    {
      contents.push_back(content);
    }
    void addWcache(std::string &wcache)
    {
      wcaches.push_back(wcache);
    }
    void addWcache(std::string &&wcache)
    {
      wcaches.push_back(wcache);
    }
    
    void setHighlightedCounter(unsigned int _highlightedCounter)
    {
      highlightedCounter = _highlightedCounter;
    }
    void setFoundCounter(unsigned int _foundCounter)
    {
      foundCounter = _foundCounter;
    }

    unsigned int getErrCode() const
    {
      return errorCode;
    }
    std::string &getErrorMsg()
    {
      return errorMsg;
    }
    std::string &getContent(unsigned int i)
    {
      if(i >= contents.size())
      {
        throw std::out_of_range("contents out of range");
      }
      return contents[i];
    }
    std::string &getWcache(unsigned int i)
    {
      if(wcaches.size() <= i)
      {
        throw std::out_of_range(">>> Index out of range");
      }
      return wcaches[i];
    }    
    unsigned int getHighlightedCounter() const
    {
      return highlightedCounter;
    }
    unsigned int getFoundCounter() const
    {
      return foundCounter;
    }
    unsigned int getContentsCount() const
    {
      return contents.size();
    }
    unsigned int getWcachesCount() const
    {
      return wcaches.size();
    }    
    ~OutDataStruct() {}
};

#endif